@@ -1,18 +1,15 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
-import base64
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import os
 from operator import itemgetter
 from collections import defaultdict
 from functools import partial
 
-from sql import Table
-from sql.aggregate import Count
+from sql import Null
 
-from ..model import ModelView, ModelStorage, ModelSQL, fields
-from ..tools import file_open, safe_eval
-from .. import backend
-from ..pyson import PYSONEncoder, CONTEXT, PYSON
+from ..model import ModelView, ModelStorage, ModelSQL, DeactivableMixin, fields
+from ..tools import file_open
+from ..pyson import PYSONDecoder, PYSON, Eval
 from ..transaction import Transaction
 from ..pool import Pool
 from ..cache import Cache
@@ -27,7 +24,7 @@ __all__ = [
 EMAIL_REFKEYS = set(('cc', 'to', 'subject'))
 
 
-class Action(ModelSQL, ModelView):
+class Action(DeactivableMixin, ModelSQL, ModelView):
     "Action"
     __name__ = 'ir.action'
     name = fields.Char('Name', required=True, translate=True)
@@ -38,7 +35,6 @@ class Action(ModelSQL, ModelView):
     groups = fields.Many2Many('ir.action-res.group', 'action', 'group',
             'Groups')
     icon = fields.Many2One('ir.ui.icon', 'Icon')
-    active = fields.Boolean('Active', select=True)
 
     @classmethod
     def __setup__(cls):
@@ -51,10 +47,6 @@ class Action(ModelSQL, ModelView):
     def default_usage():
         return None
 
-    @staticmethod
-    def default_active():
-        return True
-
     @classmethod
     def write(cls, actions, values, *args):
         pool = Pool()
@@ -90,8 +82,7 @@ class Action(ModelSQL, ModelView):
         columns.add('icon.rec_name')
         to_remove = ()
         if type_ == 'ir.action.report':
-            to_remove = ('report_content_custom', 'report_content',
-                'style_content')
+            to_remove = ('report_content_custom', 'report_content')
         elif type_ == 'ir.action.act_window':
             to_remove = ('domain', 'context', 'search_value')
         columns.difference_update(to_remove)
@@ -103,7 +94,6 @@ class ActionKeyword(ModelSQL, ModelView):
     __name__ = 'ir.action.keyword'
     keyword = fields.Selection([
             ('tree_open', 'Open tree'),
-            ('tree_action', 'Action tree'),
             ('form_print', 'Print form'),
             ('form_action', 'Action form'),
             ('form_relate', 'Form relate'),
@@ -127,10 +117,9 @@ class ActionKeyword(ModelSQL, ModelView):
 
     @classmethod
     def __register__(cls, module_name):
-        TableHandler = backend.get('TableHandler')
         super(ActionKeyword, cls).__register__(module_name)
 
-        table = TableHandler(Transaction().cursor, cls, module_name)
+        table = cls.__table_handler__(module_name)
         table.index_action(['keyword', 'model'], 'add')
 
     def get_groups(self, name):
@@ -138,7 +127,7 @@ class ActionKeyword(ModelSQL, ModelView):
 
     @classmethod
     def search_groups(cls, name, clause):
-        return [('action.groups',) + tuple(clause[1:])]
+        return [('action.' + clause[0],) + tuple(clause[1:])]
 
     @classmethod
     def validate(cls, actions):
@@ -149,13 +138,17 @@ class ActionKeyword(ModelSQL, ModelView):
     def check_wizard_model(self):
         ActionWizard = Pool().get('ir.action.wizard')
         if self.action.type == 'ir.action.wizard':
-            action_wizard, = ActionWizard.search([
+            action_wizards = ActionWizard.search([
                 ('action', '=', self.action.id),
                 ], limit=1)
-            if action_wizard.model:
-                if self.model.__name__ != action_wizard.model:
-                    self.raise_user_error('wrong_wizard_model', (
-                            action_wizard.rec_name,))
+            # could be empty when copying an action
+            if action_wizards:
+                action_wizard, = action_wizards
+                if action_wizard.model:
+                    if not str(self.model).startswith(
+                            '%s,' % action_wizard.model):
+                        self.raise_user_error('wrong_wizard_model', (
+                                action_wizard.rec_name,))
 
     @staticmethod
     def _convert_vals(vals):
@@ -170,11 +163,7 @@ class ActionKeyword(ModelSQL, ModelView):
     def models_get():
         pool = Pool()
         Model = pool.get('ir.model')
-        models = Model.search([])
-        res = []
-        for model in models:
-            res.append([model.model, model.name])
-        return res
+        return [(m.model, m.name) for m in Model.search([])]
 
     @classmethod
     def delete(cls, keywords):
@@ -232,8 +221,10 @@ class ActionKeyword(ModelSQL, ModelView):
         for action_keyword in action_keywords:
             type_ = action_keyword.action.type
             types[type_].append(action_keyword.action.id)
-        for type_, action_ids in types.iteritems():
-            keywords.extend(Action.get_action_values(type_, action_ids))
+        for type_, action_ids in types.items():
+            for value in Action.get_action_values(type_, action_ids):
+                value['keyword'] = keyword
+                keywords.append(value)
         keywords.sort(key=itemgetter('name'))
         cls._get_keyword_cache.set(key, keywords)
         return keywords
@@ -241,6 +232,7 @@ class ActionKeyword(ModelSQL, ModelView):
 
 class ActionMixin(ModelSQL):
     _order_name = 'action'
+    _action_name = 'name'
 
     @classmethod
     def __setup__(cls):
@@ -254,10 +246,10 @@ class ActionMixin(ModelSQL):
                 default_func = 'default_' + name
                 if getattr(Action, default_func, None):
                     setattr(cls, default_func,
-                        partial(ActionMixin.default_action, name))
+                        partial(ActionMixin._default_action, name))
 
     @staticmethod
-    def default_action(name):
+    def _default_action(name):
         pool = Pool()
         Action = pool.get('ir.action')
         return getattr(Action, 'default_' + name, None)()
@@ -293,14 +285,17 @@ class ActionMixin(ModelSQL):
 
     @classmethod
     def search_action(cls, name, clause):
-        return [('action.' + name,) + tuple(clause[1:])]
+        return [('action.' + clause[0],) + tuple(clause[1:])]
 
     @classmethod
     def create(cls, vlist):
         pool = Pool()
+        ModelView._fields_view_get_cache.clear()
+        ModelView._view_toolbar_get_cache.clear()
         Action = pool.get('ir.action')
         ir_action = cls.__table__()
         new_records = []
+        to_write = []
         for values in vlist:
             later = {}
             action_values = {}
@@ -313,9 +308,12 @@ class ActionMixin(ModelSQL):
             for field in later:
                 del values[field]
             action_values['type'] = cls.default_type()
-            cursor = Transaction().cursor
-            if cursor.nextid(cls._table):
-                cursor.setnextid(cls._table, cursor.currid(Action._table))
+            transaction = Transaction()
+            database = transaction.database
+            cursor = transaction.connection.cursor()
+            if database.nextid(transaction.connection, cls._table):
+                database.setnextid(transaction.connection, cls._table,
+                    database.currid(transaction.connection, Action._table))
             if 'action' not in values:
                 action, = Action.create([action_values])
                 values['action'] = action.id
@@ -325,10 +323,13 @@ class ActionMixin(ModelSQL):
             cursor.execute(*ir_action.update(
                     [ir_action.id], [action.id],
                     where=ir_action.id == record.id))
-            cursor.update_auto_increment(cls._table, action.id)
+            transaction.database.update_auto_increment(
+                transaction.connection, cls._table, action.id)
             record = cls(action.id)
             new_records.append(record)
-            cls.write([record], later)
+            to_write.extend(([record], later))
+        if to_write:
+            cls.write(*to_write)
         return new_records
 
     @classmethod
@@ -336,11 +337,15 @@ class ActionMixin(ModelSQL):
         pool = Pool()
         ActionKeyword = pool.get('ir.action.keyword')
         super(ActionMixin, cls).write(records, values, *args)
+        ModelView._fields_view_get_cache.clear()
+        ModelView._view_toolbar_get_cache.clear()
         ActionKeyword._get_keyword_cache.clear()
 
     @classmethod
     def delete(cls, records):
         pool = Pool()
+        ModelView._fields_view_get_cache.clear()
+        ModelView._view_toolbar_get_cache.clear()
         Action = pool.get('ir.action')
         actions = [x.action for x in records]
         super(ActionMixin, cls).delete(records)
@@ -360,32 +365,54 @@ class ActionMixin(ModelSQL):
                     default=default))
         return new_records
 
+    @classmethod
+    def get_groups(cls, name, action_id=None):
+        # TODO add cache
+        domain = [
+            (cls._action_name, '=', name),
+            ]
+        if action_id:
+            domain.append(('id', '=', action_id))
+        actions = cls.search(domain)
+        groups = {g.id for a in actions for g in a.groups}
+        return groups
+
 
 class ActionReport(ActionMixin, ModelSQL, ModelView):
     "Action report"
     __name__ = 'ir.action.report'
+    _action_name = 'report_name'
     model = fields.Char('Model')
     report_name = fields.Char('Internal Name', required=True)
-    report = fields.Char('Path')
+    report = fields.Char(
+        "Path",
+        states={
+            'invisible': Eval('is_custom', False),
+            },
+        depends=['is_custom'])
     report_content_custom = fields.Binary('Content')
+    is_custom = fields.Function(fields.Boolean("Is Custom"), 'get_is_custom')
     report_content = fields.Function(fields.Binary('Content',
             filename='report_content_name'),
         'get_report_content', setter='set_report_content')
-    report_content_name = fields.Function(fields.Char('Content Name',
-            on_change_with=['name', 'template_extension']),
+    report_content_name = fields.Function(fields.Char('Content Name'),
         'on_change_with_report_content_name')
     action = fields.Many2One('ir.action', 'Action', required=True,
             ondelete='CASCADE')
-    style = fields.Property(fields.Char('Style',
-            help='Define the style to apply on the report.'))
-    style_content = fields.Function(fields.Binary('Style'),
-            'get_style_content')
     direct_print = fields.Boolean('Direct Print')
+    single = fields.Boolean("Single",
+        help="Check if the template works only for one record.")
+    translatable = fields.Boolean("Translatable",
+        help="Uncheck to disable translations for this report.")
     template_extension = fields.Selection([
             ('odt', 'OpenDocument Text'),
             ('odp', 'OpenDocument Presentation'),
             ('ods', 'OpenDocument Spreadsheet'),
             ('odg', 'OpenDocument Graphics'),
+            ('txt', 'Plain Text'),
+            ('xml', 'XML'),
+            ('html', 'HTML'),
+            ('xhtml', 'XHTML'),
             ], string='Template Extension', required=True,
         translate=False)
     extension = fields.Selection([
@@ -399,6 +426,8 @@ class ActionReport(ActionMixin, ModelSQL, ModelView):
             ('doc6', 'Microsoft Word 6.0'),
             ('doc95', 'Microsoft Word 95'),
             ('docbook', 'DocBook'),
+            ('docx', 'Microsoft Office Open XML Text'),
+            ('docx7', 'Microsoft Word 2007 XML'),
             ('emf', 'Enhanced Metafile'),
             ('eps', 'Encapsulated PostScript'),
             ('gif', 'Graphics Interchange Format'),
@@ -448,10 +477,11 @@ class ActionReport(ActionMixin, ModelSQL, ModelView):
             ('xls', 'Microsoft Excel 97/2000/XP'),
             ('xls5', 'Microsoft Excel 5.0'),
             ('xls95', 'Microsoft Excel 95'),
+            ('xlsx', 'Microsoft Excel 2007/2010 XML'),
             ('xpm', 'X PixMap'),
             ], translate=False,
         string='Extension', help='Leave empty for the same as template, '
-        'see unoconv documentation for compatible format')
+        'see LibreOffice documentation for compatible format')
     module = fields.Char('Module', readonly=True, select=True)
     email = fields.Char('Email',
         help='Python dictonary where keys define "to" "cc" "subject"\n'
@@ -461,73 +491,27 @@ class ActionReport(ActionMixin, ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(ActionReport, cls).__setup__()
-        cls._sql_constraints += [
-            ('report_name_module_uniq', 'UNIQUE(report_name, module)',
-                'The internal name must be unique by module!'),
-        ]
         cls._error_messages.update({
                 'invalid_email': 'Invalid email definition on report "%s".',
                 })
 
     @classmethod
     def __register__(cls, module_name):
-        TableHandler = backend.get('TableHandler')
         super(ActionReport, cls).__register__(module_name)
 
-        cursor = Transaction().cursor
-        table = TableHandler(cursor, cls, module_name)
+        transaction = Transaction()
+        cursor = transaction.connection.cursor()
+        table = cls.__table_handler__(module_name)
         action_report = cls.__table__()
 
-        # Migration from 1.0 report_name_uniq has been removed
-        table.drop_constraint('report_name_uniq')
+        # Migration from 3.4 remove report_name_module_uniq constraint
+        table.drop_constraint('report_name_module_uniq')
 
-        # Migration from 1.0 output_format (m2o) is now extension (selection)
-        if table.column_exist('output_format'):
-            outputformat = Table('ir_action_report_outputformat')
-            cursor.execute(*action_report.join(
-                    outputformat,
-                    condition=action_report.output_format == outputformat.id)
-                .select(action_report.id,
-                    where=outputformat.format == 'pdf'))
-
-            ids = [x[0] for x in cursor.fetchall()]
-            with Transaction().set_user(0):
-                cls.write(cls.browse(ids), {'extension': 'pdf'})
-                ids = cls.search([('id', 'not in', ids)])
-                cls.write(cls.browse(ids), {'extension': 'odt'})
-
-            table.drop_column("output_format")
-            TableHandler.dropTable(cursor, 'ir.action.report.outputformat',
-                'ir_action_report_outputformat')
-
-        # Migrate from 2.0 remove required on extension
-        table.not_null_action('extension', action='remove')
+        # Migration from 4.4 replace plain extension by txt
         cursor.execute(*action_report.update(
                 [action_report.extension],
-                [''],
-                where=action_report.extension == 'odt'))
-
-        # Migration from 2.0 report_content_data renamed into
-        # report_content_custom to remove base64 encoding
-        if (table.column_exist('report_content_data')
-                and table.column_exist('report_content_custom')):
-            limit = cursor.IN_MAX
-            cursor.execute(*action_report.select(
-                    Count(action_report.id)))
-            report_count, = cursor.fetchone()
-            for offset in range(0, report_count, limit):
-                cursor.execute(*action_report.select(
-                        action_report.id, action_report.report_content_data,
-                        order_by=action_report.id,
-                        limit=limit, offset=offset))
-                for report_id, report in cursor.fetchall():
-                    if report:
-                        report = buffer(base64.decodestring(str(report)))
-                        cursor.execute(*action_report.update(
-                                [action_report.report_content_custom],
-                                [report],
-                                where=action_report.id == report_id))
-            table.drop_column('report_content_data')
+                ['txt'],
+                where=action_report.extension == 'plain'))
 
     @staticmethod
     def default_type():
@@ -541,6 +525,14 @@ class ActionReport(ActionMixin, ModelSQL, ModelView):
     def default_direct_print():
         return False
 
+    @classmethod
+    def default_single(cls):
+        return False
+
+    @classmethod
+    def default_translatable(cls):
+        return True
+
     @staticmethod
     def default_template_extension():
         return 'odt'
@@ -564,7 +556,7 @@ class ActionReport(ActionMixin, ModelSQL, ModelView):
         for report in reports:
             if report.email:
                 try:
-                    value = safe_eval(report.email, CONTEXT)
+                    value = PYSONDecoder().decode(report.email)
                 except Exception:
                     value = None
                 if isinstance(value, dict):
@@ -575,13 +567,16 @@ class ActionReport(ActionMixin, ModelSQL, ModelView):
                 else:
                     cls.raise_user_error('invalid_email', (report.rec_name,))
 
+    def get_is_custom(self, name):
+        return bool(self.report_content_custom)
+
     @classmethod
     def get_report_content(cls, reports, name):
         contents = {}
-        converter = buffer
+        converter = fields.Binary.cast
         default = None
-        format_ = Transaction().context.pop('%s.%s'
-            % (cls.__name__, name), '')
+        format_ = Transaction().context.get(
+            '%s.%s' % (cls.__name__, name), '')
         if format_ == 'size':
             converter = len
             default = 0
@@ -602,42 +597,21 @@ class ActionReport(ActionMixin, ModelSQL, ModelView):
     def set_report_content(cls, records, name, value):
         cls.write(records, {'%s_custom' % name: value})
 
+    @fields.depends('name', 'template_extension')
     def on_change_with_report_content_name(self, name=None):
-        if not self.name:
-            return
-        return ''.join([self.name, os.extsep, self.template_extension])
-
-    @classmethod
-    def get_style_content(cls, reports, name):
-        contents = {}
-        converter = buffer
-        default = None
-        format_ = Transaction().context.pop('%s.%s'
-            % (cls.__name__, name), '')
-        if format_ == 'size':
-            converter = len
-            default = 0
-        for report in reports:
-            try:
-                with file_open(report.style.replace('/', os.sep),
-                        mode='rb') as fp:
-                    data = fp.read()
-            except Exception:
-                data = None
-            contents[report.id] = converter(data) if data else default
-        return contents
+        return ''.join(
+            filter(None, [self.name, os.extsep, self.template_extension]))
 
     @classmethod
     def get_pyson(cls, reports, name):
         pysons = {}
-        encoder = PYSONEncoder()
         field = name[6:]
         defaults = {
             'email': '{}',
             }
         for report in reports:
-            pysons[report.id] = encoder.encode(safe_eval(getattr(report, field)
-                    or defaults.get(field, 'None'), CONTEXT))
+            pysons[report.id] = (getattr(report, field)
+                or defaults.get(field, 'null'))
         return pysons
 
     @classmethod
@@ -678,18 +652,19 @@ class ActionActWindow(ActionMixin, ModelSQL, ModelView):
     context = fields.Char('Context Value')
     order = fields.Char('Order Value')
     res_model = fields.Char('Model')
+    context_model = fields.Char('Context Model')
+    context_domain = fields.Char(
+        "Context Domain",
+        help="Part of the domain that will be evaluated on each refresh")
     act_window_views = fields.One2Many('ir.action.act_window.view',
             'act_window', 'Views')
     views = fields.Function(fields.Binary('Views'), 'get_views')
     act_window_domains = fields.One2Many('ir.action.act_window.domain',
         'act_window', 'Domains')
     domains = fields.Function(fields.Binary('Domains'), 'get_domains')
-    limit = fields.Integer('Limit', required=True,
-            help='Default limit for the list view')
+    limit = fields.Integer('Limit', help='Default limit for the list view')
     action = fields.Many2One('ir.action', 'Action', required=True,
             ondelete='CASCADE')
-    window_name = fields.Boolean('Window Name',
-            help='Use the action name as window name')
     search_value = fields.Char('Search Criteria',
             help='Default search criteria for the list view')
     pyson_domain = fields.Function(fields.Char('PySON Domain'), 'get_pyson')
@@ -716,21 +691,24 @@ class ActionActWindow(ActionMixin, ModelSQL, ModelView):
 
     @classmethod
     def __register__(cls, module_name):
-        cursor = Transaction().cursor
-        TableHandler = backend.get('TableHandler')
+        cursor = Transaction().connection.cursor()
         act_window = cls.__table__()
         super(ActionActWindow, cls).__register__(module_name)
 
-        table = TableHandler(cursor, cls, module_name)
-
-        # Migration from 2.0: new search_value format
-        cursor.execute(*act_window.update(
-                [act_window.search_value], ['[]'],
-                where=act_window.search_value == '{}'))
+        table = cls.__table_handler__(module_name)
 
         # Migration from 3.0: auto_refresh removed
         table.drop_column('auto_refresh')
 
+        # Migration from 4.0: window_name removed
+        table.drop_column('window_name')
+
+        # Migration from 4.2: remove required on limit
+        table.not_null_action('limit', 'remove')
+        cursor.execute(*act_window.update(
+                [act_window.limit], [Null],
+                where=act_window.limit == 0))
+
     @staticmethod
     def default_type():
         return 'ir.action.act_window'
@@ -739,14 +717,6 @@ class ActionActWindow(ActionMixin, ModelSQL, ModelView):
     def default_context():
         return '{}'
 
-    @staticmethod
-    def default_limit():
-        return 0
-
-    @staticmethod
-    def default_window_name():
-        return True
-
     @staticmethod
     def default_search_value():
         return '[]'
@@ -797,7 +767,7 @@ class ActionActWindow(ActionMixin, ModelSQL, ModelView):
                 if not domain:
                     continue
                 try:
-                    value = safe_eval(domain, CONTEXT)
+                    value = PYSONDecoder().decode(domain)
                 except Exception:
                     cls.raise_user_error('invalid_domain', {
                             'domain': domain,
@@ -829,7 +799,7 @@ class ActionActWindow(ActionMixin, ModelSQL, ModelView):
         for action in actions:
             if action.context:
                 try:
-                    value = safe_eval(action.context, CONTEXT)
+                    value = PYSONDecoder().decode(action.context)
                 except Exception:
                     cls.raise_user_error('invalid_context', {
                             'context': action.context,
@@ -860,24 +830,21 @@ class ActionActWindow(ActionMixin, ModelSQL, ModelView):
             for view in self.act_window_views]
 
     def get_domains(self, name):
-        encoder = PYSONEncoder()
-        return [(domain.name,
-                encoder.encode(safe_eval(domain.domain or '[]', CONTEXT)))
+        return [(domain.name, domain.domain or '[]', domain.count)
             for domain in self.act_window_domains]
 
     @classmethod
     def get_pyson(cls, windows, name):
         pysons = {}
-        encoder = PYSONEncoder()
         field = name[6:]
         defaults = {
             'domain': '[]',
             'context': '{}',
-            'search_value': '{}',
+            'search_value': '[]',
             }
         for window in windows:
-            pysons[window.id] = encoder.encode(safe_eval(getattr(window, field)
-                    or defaults.get(field, 'None'), CONTEXT))
+            pysons[window.id] = (getattr(window, field)
+                or defaults.get(field, 'null'))
         return pysons
 
     @classmethod
@@ -893,10 +860,9 @@ class ActionActWindow(ActionMixin, ModelSQL, ModelView):
         return Action.get_action_values(cls.__name__, [action_id])[0]
 
 
-class ActionActWindowView(ModelSQL, ModelView):
+class ActionActWindowView(DeactivableMixin, ModelSQL, ModelView):
     "Action act window view"
     __name__ = 'ir.action.act_window.view'
-    _rec_name = 'view'
     sequence = fields.Integer('Sequence', required=True)
     view = fields.Many2One('ir.ui.view', 'View', required=True,
             ondelete='CASCADE')
@@ -908,15 +874,6 @@ class ActionActWindowView(ModelSQL, ModelView):
         super(ActionActWindowView, cls).__setup__()
         cls._order.insert(0, ('sequence', 'ASC'))
 
-    @classmethod
-    def __register__(cls, module_name):
-        TableHandler = backend.get('TableHandler')
-        super(ActionActWindowView, cls).__register__(module_name)
-        table = TableHandler(Transaction().cursor, cls, module_name)
-
-        # Migration from 1.0 remove multi
-        table.drop_column('multi')
-
     @classmethod
     def create(cls, vlist):
         pool = Pool()
@@ -937,29 +894,83 @@ class ActionActWindowView(ModelSQL, ModelView):
         pool.get('ir.action.keyword')._get_keyword_cache.clear()
 
 
-class ActionActWindowDomain(ModelSQL, ModelView):
+class ActionActWindowDomain(DeactivableMixin, ModelSQL, ModelView):
     "Action act window domain"
     __name__ = 'ir.action.act_window.domain'
     name = fields.Char('Name', translate=True)
     sequence = fields.Integer('Sequence', required=True)
     domain = fields.Char('Domain')
+    count = fields.Boolean('Count')
     act_window = fields.Many2One('ir.action.act_window', 'Action',
         select=True, required=True, ondelete='CASCADE')
-    active = fields.Boolean('Active')
 
     @classmethod
     def __setup__(cls):
         super(ActionActWindowDomain, cls).__setup__()
         cls._order.insert(0, ('sequence', 'ASC'))
+        cls._error_messages.update({
+                'invalid_domain': ('Invalid domain or search criteria '
+                    '"%(domain)s" on action "%(action)s".'),
+                })
 
-    @staticmethod
-    def default_active():
-        return True
+    @classmethod
+    def default_count(cls):
+        return False
+
+    @classmethod
+    def validate(cls, actions):
+        super(ActionActWindowDomain, cls).validate(actions)
+        cls.check_domain(actions)
+
+    @classmethod
+    def check_domain(cls, actions):
+        for action in actions:
+            if not action.domain:
+                continue
+            try:
+                value = PYSONDecoder().decode(action.domain)
+            except Exception:
+                value = None
+            if isinstance(value, PYSON):
+                if not value.types() == set([list]):
+                    value = None
+            elif not isinstance(value, list):
+                value = None
+            else:
+                try:
+                    fields.domain_validate(value)
+                except Exception:
+                    value = None
+            if value is None:
+                cls.raise_user_error('invalid_domain', {
+                        'domain': action.domain,
+                        'action': action.rec_name,
+                        })
+
+    @classmethod
+    def create(cls, vlist):
+        pool = Pool()
+        domains = super(ActionActWindowDomain, cls).create(vlist)
+        pool.get('ir.action.keyword')._get_keyword_cache.clear()
+        return domains
+
+    @classmethod
+    def write(cls, domains, values, *args):
+        pool = Pool()
+        super(ActionActWindowDomain, cls).write(domains, values, *args)
+        pool.get('ir.action.keyword')._get_keyword_cache.clear()
+
+    @classmethod
+    def delete(cls, domains):
+        pool = Pool()
+        super(ActionActWindowDomain, cls).delete(domains)
+        pool.get('ir.action.keyword')._get_keyword_cache.clear()
 
 
 class ActionWizard(ActionMixin, ModelSQL, ModelView):
     "Action wizard"
     __name__ = 'ir.action.wizard'
+    _action_name = 'wiz_name'
     wiz_name = fields.Char('Wizard name', required=True)
     action = fields.Many2One('ir.action', 'Action', required=True,
             ondelete='CASCADE')
@@ -982,7 +993,3 @@ class ActionURL(ActionMixin, ModelSQL, ModelView):
     @staticmethod
     def default_type():
         return 'ir.action.url'
-
-    @staticmethod
-    def default_target():
-        return 'new'
