@@ -1,22 +1,22 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 
-__all__ = ['Wizard', 'StateView', 'StateTransition', 'StateAction', 'Button']
+__all__ = ['Wizard',
+    'StateView', 'StateTransition', 'StateAction', 'StateReport',
+    'Button']
 
-try:
-    import simplejson as json
-except ImportError:
-    import json
+import json
 import copy
 
 from trytond.pool import Pool, PoolBase
 from trytond.transaction import Transaction
 from trytond.error import WarningErrorMixin
 from trytond.url import URLMixin
-from trytond.protocols.jsonrpc import object_hook, JSONEncoder
+from trytond.protocols.jsonrpc import JSONDecoder, JSONEncoder
 from trytond.model.fields import states_validate
 from trytond.pyson import PYSONEncoder
 from trytond.rpc import RPC
+from trytond.exceptions import UserError
 
 
 class Button(object):
@@ -66,25 +66,40 @@ class StateView(State):
         assert len(self.buttons) == len(set(b.state for b in self.buttons))
         assert len([b for b in self.buttons if b.default]) <= 1
 
-    def get_view(self):
+    def get_view(self, wizard, state_name):
         '''
         Returns the view definition
         '''
         Model_ = Pool().get(self.model_name)
         ModelData = Pool().get('ir.model.data')
-        module, fs_id = self.view.split('.')
-        view_id = ModelData.get_id(module, fs_id)
+        if self.view:
+            module, fs_id = self.view.split('.')
+            view_id = ModelData.get_id(module, fs_id)
+        else:
+            view_id = None
         return Model_.fields_view_get(view_id=view_id, view_type='form')
 
     def get_defaults(self, wizard, state_name, fields):
         '''
         Returns defaults values for the fields
         '''
-        Model_ = Pool().get(self.model_name)
+        pool = Pool()
+        Model_ = pool.get(self.model_name)
         defaults = Model_.default_get(fields)
         default = getattr(wizard, 'default_%s' % state_name, None)
         if default:
             defaults.update(default(fields))
+            for field_name, value in list(defaults.items()):
+                if '.' in field_name:
+                    continue
+                field = Model_._fields[field_name]
+                field_rec_name = field_name + '.rec_name'
+                if (value
+                        and field._type == 'many2one'
+                        and field_rec_name not in defaults):
+                    Target = pool.get(field.model_name)
+                    if 'rec_name' in Target._fields:
+                        defaults[field_rec_name] = Target(value).rec_name
         return defaults
 
     def get_buttons(self, wizard, state_name):
@@ -142,6 +157,27 @@ class StateAction(StateTransition):
         return Action.get_action_values(action.type, [action.id])[0]
 
 
+class StateReport(StateAction):
+    'An report state of a wizard'
+
+    def __init__(self, report_name):
+        super(StateReport, self).__init__(None)
+        self.report_name = report_name
+
+    def get_action(self):
+        'Return report definition'
+        pool = Pool()
+        ActionReport = pool.get('ir.action.report')
+        Action = pool.get('ir.action')
+        action_reports = ActionReport.search([
+                ('report_name', '=', self.report_name),
+                ])
+        assert action_reports, '%s not found' % self.report_name
+        action_report = action_reports[0]
+        action = action_report.action
+        return Action.get_action_values(action.type, [action.id])[0]
+
+
 class Wizard(WarningErrorMixin, URLMixin, PoolBase):
     start_state = 'start'
     end_state = 'end'
@@ -152,7 +188,7 @@ class Wizard(WarningErrorMixin, URLMixin, PoolBase):
         cls.__rpc__ = {
             'create': RPC(readonly=False),
             'delete': RPC(readonly=False),
-            'execute': RPC(readonly=False),
+            'execute': RPC(readonly=False, check_access=False),
             }
         cls._error_messages = {}
 
@@ -180,10 +216,35 @@ class Wizard(WarningErrorMixin, URLMixin, PoolBase):
         Translation.register_wizard(cls, module_name)
         Translation.register_error_messages(cls, module_name)
 
+    @classmethod
+    def check_access(cls):
+        pool = Pool()
+        ModelAccess = pool.get('ir.model.access')
+        ActionWizard = pool.get('ir.action.wizard')
+        User = pool.get('res.user')
+        context = Transaction().context
+
+        if Transaction().user == 0:
+            return
+
+        model = context.get('active_model')
+        if model:
+            ModelAccess.check(model, 'read')
+        groups = set(User.get_groups())
+        wizard_groups = ActionWizard.get_groups(cls.__name__,
+            action_id=context.get('action_id'))
+        if wizard_groups:
+            if not groups & wizard_groups:
+                raise UserError('Calling wizard %s is not allowed!'
+                    % cls.__name__)
+        elif model:
+            ModelAccess.check(model, 'write')
+
     @classmethod
     def create(cls):
         "Create a session"
         Session = Pool().get('ir.session.wizard')
+        cls.check_access()
         return (Session.create([{}])[0].id, cls.start_state, cls.end_state)
 
     @classmethod
@@ -215,10 +276,11 @@ class Wizard(WarningErrorMixin, URLMixin, PoolBase):
                 - ``defaults``: a dictionary with default values
                 - ``buttons``: a list of buttons
         '''
+        cls.check_access()
         wizard = cls(session_id)
-        for key, values in data.iteritems():
+        for key, values in data.items():
             record = getattr(wizard, key)
-            for field, value in values.iteritems():
+            for field, value in values.items():
                 if field == 'id':
                     continue
                 setattr(record, field, value)
@@ -232,9 +294,9 @@ class Wizard(WarningErrorMixin, URLMixin, PoolBase):
         result = {}
 
         if isinstance(state, StateView):
-            view = state.get_view()
+            view = state.get_view(self, state_name)
             defaults = state.get_defaults(self, state_name,
-                view['fields'].keys())
+                list(view['fields'].keys()))
             buttons = state.get_buttons(self, state_name)
             result['view'] = {
                 'fields_view': view,
@@ -264,9 +326,8 @@ class Wizard(WarningErrorMixin, URLMixin, PoolBase):
         Session = pool.get('ir.session.wizard')
         self._session_id = session_id
         session = Session(session_id)
-        data = json.loads(session.data.encode('utf-8'),
-            object_hook=object_hook)
-        for state_name, state in self.states.iteritems():
+        data = json.loads(session.data, object_hook=JSONDecoder())
+        for state_name, state in self.states.items():
             if isinstance(state, StateView):
                 Target = pool.get(state.model_name)
                 data.setdefault(state_name, {})
@@ -276,11 +337,11 @@ class Wizard(WarningErrorMixin, URLMixin, PoolBase):
         "Save the session in database"
         Session = Pool().get('ir.session.wizard')
         data = {}
-        for state_name, state in self.states.iteritems():
+        for state_name, state in self.states.items():
             if isinstance(state, StateView):
                 data[state_name] = getattr(self, state_name)._default_values
         session = Session(self._session_id)
-        data = json.dumps(data, cls=JSONEncoder)
+        data = json.dumps(data, cls=JSONEncoder, separators=(',', ':'))
         if data != session.data.encode('utf-8'):
             Session.write([session], {
                     'data': data,
